"""Get a valid pyppeteer browser.

from exttend-euserv, extracted from deepl_tr_pp.

rid of LOOP and BROWSER
"""
# pylint: disable=too-many-arguments, too-many-locals, too-many-branches, too-many-statements

from typing import (
    Any,
    Optional,
    Union,
)

from pathlib import Path
import asyncio
import platform
import tempfile

import pyppeteer
from pyppeteer import launch

import logzero
from logzero import logger

from get_ppbrowser.config import Settings

LOOP = asyncio.get_event_loop()

CONFIG = Settings()  # CONFIG = Settings(env=dotenv.find_dotenv())
HEADFUL = CONFIG.headful
DEBUG = CONFIG.debug
PROXY = "" if CONFIG.proxy is None else CONFIG.proxy

logger.info(" HEADFUL: %s", HEADFUL)
logger.info(" DEBUG: %s", DEBUG)
logger.info(" PROXY: %s", PROXY)
if DEBUG:
    logzero.loglevel(10)


# fmt: off
async def get_ppbrowser(
        headless: bool = not HEADFUL,
        proxy: Optional[str] = PROXY,
        executable_path: Optional[Union[str, Path]] = None,
        **kwargs: Any,
) -> pyppeteer.browser.Browser:
    # fmt: on
    """Get a puppeeter browser.

    headless=not HEADFUL; proxy: str = PROXY
    """
    # half-hearted attempt to use an existing chrome
    if Path(r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe").exists():
        executable_path = r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe"
    elif Path(r"D:\Program Files (x86)\Google\Chrome\Application\chrome.exe").exists():
        executable_path = r"D:\Program Files (x86)\Google\Chrome\Application\chrome.exe"

    _ = """
    # devtools = False
    if not headless:  # if headless is False
        devtools = True  # pop devtools, auto headless=False
    # """

    # tempdir = Path("/tmp" if platform.system() == "Darwin" else tempfile.gettempdir())
    # mkdir a random dir for each session

    # tfile = tempfile.NamedTemporaryFile()
    # tfile.close()

    tname = tempfile.NamedTemporaryFile().name
    tempdir = Path("/tmp" if platform.system() == "Darwin" else tname)
    tempdir.mkdir(exist_ok=True)

    try:
        browser = await launch(
            args=[
                "--disable-infobars",
                "--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36",
                "--window-size=1440x900",
                # "--autoClose=False",
                f"--proxy-server={proxy}",
                "--disable-popup-blocking",
                "--ignoreHTTPSErrors"
            ],
            ignoreDefaultArgs=[
                "--enable-automation",  # set window.navigator.webdriver to undefined
            ],
            executablePath=executable_path,  # use chrome
            # autoClose=False,
            headless=headless,
            # devtools=devtools,  # replace headless
            dumpio=True,
            # userDataDir=".",
            userDataDir=tempdir,
        )
    except Exception as exc:
        logger.error("get_ppbrowser exc: %s", exc)
        raise
    # page = await browser.newPage()
    # await page.goto(url)
    # logger.debug("page.goto deepl time: %.2f s", default_timer() - then)
    return browser

_ = """
try:
    BROWSER = LOOP.run_until_complete(get_ppbrowser(not HEADFUL))
except Exception as exc:
    logger.error(" Unable to pyppeteer.launch exc: %s", exc)
    logger.info(
        "\n\t%s",
        r"Possible cause: abnormal exit from a previous session. Try `taskkill /f /im chrome.exe`",
    )
    logger.warning(" %s", "Note that this will also kill your chrome browser.")
    raise SystemExit(1)
# """


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['get_ppbrowser']

package_data = \
{'': ['*']}

install_requires = \
['logzero>=1.6.3,<2.0.0',
 'pydantic>=1.7.3,<2.0.0',
 'pyppeteer2>=0.2.2,<0.3.0',
 'pytest-asyncio>=0.14.0,<0.15.0',
 'python-dotenv>=0.15.0,<0.16.0']

setup_kwargs = {
    'name': 'get-ppbrowser',
    'version': '0.1.3',
    'description': 'Create a valid pyppeteer browser',
    'long_description': '# get-ppbrowser\n[![python](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)[![build-and-pytest](https://github.com/ffreemt/get-ppbrowser/actions/workflows/build-and-pytest.yml/badge.svg)](https://github.com/ffreemt/get-ppbrowser/actions/workflows/build-and-pytest.yml)[![CodeQL](https://github.com/ffreemt/get-ppbrowser/actions/workflows/codeql-analysis.yml/badge.svg)](https://github.com/ffreemt/get-ppbrowser/actions/workflows/codeql-analysis.yml)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/get-ppbrowser.svg)](https://badge.fury.io/py/get-ppbrowser)\n\nInstantiate a pyppeteer browser object.\n\n## Instalation\n```bash\npip install get-ppbrowser\n# pip  install get-ppbrowser -U  # to upgrade\n```\nor\n```bash\npoetry add get-ppbrowser\n# poetry add get-ppbrowser@latest  # to upgrade\n```\n\n## Usage\n```python\nfrom get_ppbrowser import get_ppbrowser\n\nbrowser = await get_ppbrowser()\npage = await browser.newPage()\nawait page.goto("http://www.example.com")\n```\n\n### PPBROWSER_HEADFUL, PPBROWSER_DEBUG, PPBROWSER_PROXY\nEnvironment variables can be set with, for example, in Windows\n\n```bash\nset PPBROWSER_HEADFUL=1\n```\n\nor in Linux/iOS\n```bash\nexport PPBROWSER_HEADFUL=1\n```\nor in `python`\n\n```python\nimport os\n\nos.environ["PPBROWSER_HEADFUL"] = "1"\n```\n',
    'author': 'freemt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ffreemt/get-ppbrowser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Configure params DEBUG HEADFUL PROXY."""
from typing import Optional

from pathlib import Path

# import dotenv
from pydantic import BaseSettings, AnyUrl  # pylint: disable=no-name-in-module

from logzero import logger


class Settings(BaseSettings):  # pylint: disable=too-few-public-methods
    """Configure params DEBUG HEADFUL PROXY."""

    debug: bool = False
    headful: bool = False
    proxy: Optional[AnyUrl] = None

    class Config:  # pylint: disable=too-few-public-methods
        """Config."""

        env_prefix = "PPBROWSER_"
        # extra = "allow"
        env_file = ".env"
        env_file_encoding = "utf-8"  # pydantic doc

        logger.info(
            "env_prefix: %s, env_file: %s", env_prefix, Path(env_file).resolve()
        )
